#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::BuildTransportLayer
# state        public
#----------------------------------------------------------------------------------------
# description  test plan for BuildTransportLayer.pm
#========================================================================================
package DE_EPAGES::DHLIntraship::t::BuildTransportLayer;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;
use Test::More;
use Test::Exception;

sub Main {
    testInTestEnvironment();
    testInLiveEnvironment();
    testWithWrongURL();
    done_testing();
}

sub testInTestEnvironment {
    my $subTestName = (caller(0))[3];

    my $hLoginData = {
        'Realm' => 'CIG Sandbox Services',
        'UserName' => 'abc',
        'Password' => 'def',
        'EndPointURL' => 'http://dummy.URL.com'
    };
    my $WebServiceClient = _GetWebServiceClient($hLoginData);
    my $SoapOperationTestName = 'SoapOperationNameDummy';
    my $SoapActionNameInUserAgentDefHeadersExpected = '"urn:SoapOperationNameDummy"';
    $WebServiceClient->setSOAPOperationName($SoapOperationTestName);

    my $TransportLayer = $WebServiceClient->createTransportLayer();

    is($TransportLayer->{'addrs'}[0], $hLoginData->{'EndPointURL'}, "$subTestName - check endPointURL in transport layer - string of endpoint url");
    my $aLoginDataExpected = [$hLoginData->{'UserName'}, $hLoginData->{'Password'}];
    my $aLoginDataTransportLayer = $TransportLayer->{'user_agent'}->{'basic_authentication'}->{'dummy.url.com:443'}->{ $hLoginData->{'Realm'} };
    is_deeply($aLoginDataTransportLayer, $aLoginDataExpected, "$subTestName - check LoginData in transport layer - base auth login data");

    my $SoapActionNameInUserAgentDefHeaders = $TransportLayer->{'user_agent'}->{'def_headers'}->{'soapaction'};
    is($SoapActionNameInUserAgentDefHeaders, $SoapActionNameInUserAgentDefHeadersExpected,"$subTestName - SOAP Operation Name Test in Def-Headers of User Agent - name of the soap operation");
}

sub testInLiveEnvironment {
    my $subTestName = (caller(0))[3];

    my $hLoginData = {
        'Realm' => 'CIG Production Services',
        'UserName' => 'abc',
        'Password' => 'def',
        'EndPointURL' => 'http://dummy.URL.com'
    };
    my $WebServiceClient = _GetWebServiceClient($hLoginData);
    my $SoapOperationTestName = 'SoapOperationNameDummy';
    my $SoapActionNameInUserAgentDefHeadersExpected = '"urn:SoapOperationNameDummy"';
    $WebServiceClient->setSOAPOperationName($SoapOperationTestName);

    my $TransportLayer = $WebServiceClient->createTransportLayer();

    is($TransportLayer->{'addrs'}[0], $hLoginData->{'EndPointURL'}, "$subTestName - check endPointURL in transport layer - string of endpoint url");
    my $aLoginDataExpected = [$hLoginData->{'UserName'}, $hLoginData->{'Password'}];
    my $aLoginDataTransportLayer = $TransportLayer->{'user_agent'}->{'basic_authentication'}->{'dummy.url.com:443'}->{ $hLoginData->{'Realm'} };
    is_deeply($aLoginDataTransportLayer, $aLoginDataExpected, "$subTestName - check LoginData in transport layer - base auth login data");

    my $SoapActionNameInUserAgentDefHeaders = $TransportLayer->{'user_agent'}->{'def_headers'}->{'soapaction'};
    is($SoapActionNameInUserAgentDefHeaders, $SoapActionNameInUserAgentDefHeadersExpected,"$subTestName - SOAP Operation Name Test in Def-Headers of User Agent - name of the soap operation");
}

sub testWithWrongURL {
    my $subTestName = (caller(0))[3];

    my $hLoginData = {
        'Realm' => 'Dummy Realm',
        'UserName' => 'abc',
        'Password' => 'def',
        'EndPointURL' => 'dummy.URL.com' # URL without protocol (http:)
    };

    my $WebServiceClient = _GetWebServiceClient($hLoginData);
    my $SoapOperationTestName = 'SoapOperationNameDummy';
    $WebServiceClient->setSOAPOperationName($SoapOperationTestName);

    throws_ok(
        sub {$WebServiceClient->createTransportLayer()},
        qr/EndPointURL has to begin with protocol/ ,
        "$subTestName - If URL has no protocol (http) - ePages Error"
    );
}

sub _GetWebServiceClient {
    my ($hLoginData) = @_;
    my $WebServiceClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new($hLoginData);
    return $WebServiceClient;
}

RunScript( Sub => \&Main );